#!/usr/bin/env perl
use strict;
use warnings;

use Test::More;

eval { 
    # autogenerated files are excluded
    # use strict is implied by Moose & Co.
    require Test::Kwalitee; Test::Kwalitee->import(
        tests => [qw( 
            -use_strict
            -has_manifest
            -has_test_pod_coverage
        )],
    );
};

plan skip_all => 'Test::Kwalitee not installed; skipping' if $@;
